//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------------
// $Id: signals.cxx,v 1.1 2002/11/28 17:50:59 loreti Exp $
// -------------------------------------------------------------------
// Implementation of the procedures defined in signals.hh
// -------------------------------------------------------------------

#include "signals.hh"

// Our own interface with "sigaction".  See: W. Richard Stevens, "Unix
// Network Programming", Vol. 1, pages 120 and following for details;
// this is essentially the (obsolete) procedure "signal" that predated
// POSIX.1 .

// Under Solaris 2.6 "sigaction" does not work; but there is a working
// implementation of "signal".

#if defined( __sun__ )

namespace MySignal {
  sigFunction * mySignal(
    const int     sigNr,
    sigFunction * fun
  ) {
    return signal(sigNr, fun);
  }
} // namespace

#else

namespace MySignal {
  sigFunction * mySignal(
    const int     sigNr,
    sigFunction * fun
  ) {
    struct sigaction action, oldAction;

    //  - Declare "fun" as handler for "sigNr": no additional signals
    //    to be blocked when our signal handler is called;
    //  - patch for SA_INTERRUPT (bad behavior of older SunOS 4.x);
    //  - patch for SA_RESTART (SVR4, 4.4BSD).

    sigemptyset(&action.sa_mask);
    action.sa_handler = fun;
    action.sa_flags = 0;

    if (sigNr == SIGALRM) {
#if defined(SA_INTERRUPT)
      action.sa_flags |= SA_INTERRUPT;
#endif
    } else {
#if defined(SA_RESTART)
      action.sa_flags |= SA_RESTART;
#endif
    }

    if (sigaction(sigNr, &action, &oldAction) < 0) {
      return SIG_ERR;
    } else {
      return oldAction.sa_handler;
    }
  } // mySignal
} // namespace

#endif // __sun__
