//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------------
// $Id: toyclient.cxx,v 1.1 2002/11/28 17:50:59 loreti Exp $
// -------------------------------------------------------------------
// A toy implementation of a client that connects to the server
// defined in "ipcserver.cxx" (see there for more explanations).
// -------------------------------------------------------------------

#include <iostream>
#include <stdexcept>
#include <cstdlib>
#include "signals.hh"
#include "client.hh"

using std::cerr;

int main(
  int    argc,
  char * argv[]
) {
  // Command line arguments

  if (argc != 3) {
    cerr << "Usage: " << argv[0] << " <host> <port>\n";
    return EXIT_SUCCESS;
  }

  // Signal handlers

  if (MySignal::mySignal(SIGPIPE, SIG_IGN) == SIG_ERR) {
    cerr << "Couldn't establish SIGPIPE handler\n";
    return EXIT_FAILURE;
  }

  try {
    // Initializes a connecting socket

    Client c(AF_INET, SOCK_STREAM, argv[1], atoi(argv[2]));

    // - Gets the 'welcome' message from the server (the '-1' is due
    //   to the terminating line feed received in the buffer);
    // - Loops until an end-of-file is received in input, or a
    //   connection error is detected on the socket.

    static const size_t BUFLEN = 512;
    char buffer[BUFLEN];

    c.sockGets(buffer, BUFLEN - 1);

    while (c.isConnected()) {
      std::cout << "Server: " << buffer << std::endl;

      std::string s;
      if (!std::getline(std::cin, s)) break;

      s += "\n";
      if (c.sockPuts(s.c_str()) > 0) {
        c.sockGets(buffer, BUFLEN - 1);
      }
    }
  } catch (const std::exception & e) {
    cerr << "Exception - " << e.what() << '\n';
  }
}
