#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>

#if defined(__sun__)
  int inet_aton(const char *, struct in_addr *);
#endif /* __sun__ */

/**
 | Defined constants.
 |
 | - HOST:   default host of the server program (name or numbers)
 | - PORT:   default value of the well-known port used
**/

#define HOST   "mlinux.pd.infn.it"
#define PORT          1234

/**
 | Local procedures
**/

static struct in_addr *atoaddr(const char *);
static void            usage(const char *);

int main(
  int   argc,
  char *argv[]
) {
  int             sockfd;
  short int       port;
  struct in_addr *addr;

  /**
   | Tries to decode the (optional) port address contained in argv[1];
   | the arguments are parsed in a quick and dirty way.
  **/

  if (argc > 3) {
    usage(argv[0]);
    return EXIT_FAILURE;
  }

  addr = argc > 1 ? atoaddr(argv[1]) : atoaddr(HOST);
  port = argc > 2 ? atoi(argv[2])    : PORT;

  if (addr == 0) {
    usage(argv[0]);
    return EXIT_FAILURE;
  }

  if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("socket");
  } else {
    struct sockaddr_in servaddr;

    memset(&servaddr, 0, sizeof(servaddr));
    servaddr.sin_family      = AF_INET;
    servaddr.sin_port        = htons(port);
    servaddr.sin_addr.s_addr = addr->s_addr;

    if (connect(sockfd, &servaddr, sizeof(servaddr)) < 0) {
      perror("connect");
    } else {
      int n;
      char buffer[] = "Hello";
      char *pc;
      unsigned char got;

      /**
       | Sends to the server the characters in the string "buffer";
       | then closes the connection.
      **/

      for (pc = buffer;   *pc;   ++pc) {
        do {
          n = write(sockfd, pc, 1);
        } while (n < 0   &&   errno == EINTR);
        if (n != 1) {
          if (n < 0) perror("write");
          if (close(sockfd) != 0) perror("close");
          break;
        }
        do {
          n = read(sockfd, &got, 1);
        } while (n < 1   &&   errno == EINTR);
        if (n != 1) {
          if (n < 0) perror("read");
          if (close(sockfd) != 0) perror("close");
          break;
        }
        printf("Client: sent \"%c\", got %d\n", *pc, (int) got);
      }
    }
    if (close(sockfd) != 0) perror("close");
  }
  return EXIT_SUCCESS;
}

static struct in_addr *atoaddr(
  const char *input
) {
  static struct in_addr  address;
  struct hostent        *host;

  /**
   | Tries to decode "input" as number-and dots notation; if inet_aton
   | returns error (i.e. zero) tries to decode "input" as an
   | alphanumeric identifier.
  **/

  if (inet_aton(input, &address) != 0) return &address;
  if ((host = gethostbyname(input)) == 0) return 0;
  return (struct in_addr *) *host->h_addr_list;
}

static void usage(
  const char *pc
) {
  printf("Usage: %s [host [port]]\n", pc);
  printf("\tThe host (numbers-and-dots or alphanumeric) defaults to \"%s\";\n",
         HOST);
  printf("\tthe port defaults to %d.\n", PORT);
}
#if defined(__sun__)

int inet_aton(
  const char     *pc,
  struct in_addr *pin
) {
  in_addr_t addr = inet_addr(pc);
  if (addr == (in_addr_t) -1) {
    return 0;
  } else {
    pin->s_addr = addr;
    return 1;
  }
}
  
#endif /* __sun__ */
