/* $Id: toyserver.c,v 1.3 2006/09/11 12:48:15 loreti Exp $ */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "sockhelp.h"

#if defined(DAEMON)
  #include "daemonize.h"

  char progName[FILENAME_MAX];

  static void baseName(char[], const char[]);
#endif

#define BUFFER_LEN 1024

int listensock  = -1;
int connectsock = -1;

static void sig_chld(const int);

int main(
  int   argc,
  char *argv[]
) {
  int  sock;
  int  connected;
  int  port;
  char buffer[BUFFER_LEN];

#if defined(DAEMON)
  baseName(progName, argv[0]);
#endif

/**
 | - Checks the program arguments;
 | - ignores SIGPIPE; this is usually a good idea, since the default
 |   behavior is to terminate the application.  SIGPIPE is sent when
 |   you try to write to an unconnected socket; you should check your
 |   return codes to make sure you catch this error!
 | - establishes the signal handler for SIGCHLD.
**/

  if (argc != 2) {
    fprintf(stderr, "Usage:\t%s <port>\n", argv[0]);
    fputs("\t- <port> is the service name or the port number to listen to.\n",
          stderr);
    return EXIT_FAILURE;
  }

  if (my_signal(SIGPIPE, SIG_IGN) == SIG_ERR) {
    perror("sigaction(SIGPIPE)");
    return EXIT_FAILURE;
  }

  if (my_signal(SIGCHLD, sig_chld) == SIG_ERR) {
    perror("sigaction(SIGCHLD)");
    return EXIT_FAILURE;
  }

/**
 | - Tries to decode <port> as a service name or as a number;
 | - calls get_connection, that forks a new child for every
 |   established connection and then returns;
 | - sends the first message to the connected client.
**/

  if ((port = atoport(argv[1], "tcp")) == -1) {
    fprintf(stderr, "Unable to find service \"%s\"\n", argv[1]);
    return EXIT_FAILURE;
  }

#if defined(DAEMON)
  daemonize(progName);
#endif

  connectsock = sock = get_connection(SOCK_STREAM, port, &listensock);
  sock_puts(sock, "Welcome to the upper case server.\n");
  for (connected = 1;   connected;   ) {

/**
 | Read input, and echo back the same in uppercase
 | (plus a terminating line feed)
**/

    if (sock_gets(sock, buffer, BUFFER_LEN) < 0) {
      connected = 0;
    } else {
      char *pc;

      for (pc = buffer;   *pc != '\0';   pc++) {
        *pc = toupper((unsigned char) *pc);
      }
      *pc++ = '\n';
      *pc   = '\0';

      if (sock_puts(sock, buffer) < 0) {
        connected = 0;
      }
    }
  }

  close(sock);
  return EXIT_SUCCESS;
}

static void sig_chld(
 const int sig
) {

/**
 | This procedure waits for all forked children, so that they
 | don't become zombies
**/

  pid_t pid;
  int   status;

  while ((pid = waitpid(-1, &status, WNOHANG)) > 0) { }
}

#if defined(DAEMON)
static void baseName(
  char       name[],
  const char pc[]
){

/**
 | Strips the (eventual) path information from the program name (in
 | pc[], i.e. argv[0]), storing the result into name[]; if no file
 | name is given, returns an empty string.
**/

  char *p = strrchr(pc, '/');
  if (p == 0) {
    strcpy(name, pc);
  } else {
    strcpy(name, ++p);
  }
}
#endif /* DAEMON */
